"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var Support_1 = require("./Support");
function makeTrie() {
    var arr = [];
    for (var i = 0; i < 128; i++) {
        arr.push(undefined);
    }
    return { count: 0, arr: arr };
}
function lookup(t, seq, i) {
    var first = seq.charCodeAt(i);
    if (first >= 128) {
        first = 0;
    }
    if (i >= seq.length - 1) {
        if (typeof t !== "object") {
            return Support_1.panic("Malformed trie");
        }
        var n = t.arr[first];
        if (typeof n === "object") {
            return Support_1.panic("Malformed trie");
        }
        if (n === undefined) {
            return undefined;
        }
        return n / t.count;
    }
    var st = t.arr[first];
    if (st === undefined) {
        return undefined;
    }
    if (typeof st !== "object") {
        return Support_1.panic("Malformed trie");
    }
    return lookup(st, seq, i + 1);
}
function increment(t, seq, i) {
    var first = seq.charCodeAt(i);
    if (first >= 128) {
        first = 0;
    }
    if (i >= seq.length - 1) {
        if (typeof t !== "object") {
            return Support_1.panic("Malformed trie");
        }
        var n = t.arr[first];
        if (typeof n === "object") {
            return Support_1.panic("Malformed trie");
        }
        if (n === undefined) {
            n = 0;
        }
        t.arr[first] = n + 1;
        t.count += 1;
        return;
    }
    var st = t.arr[first];
    if (st === undefined) {
        t.arr[first] = st = makeTrie();
    }
    if (typeof st !== "object") {
        return Support_1.panic("Malformed trie");
    }
    return increment(st, seq, i + 1);
}
function train(filename, depth) {
    var contents = fs.readFileSync(filename).toString();
    var lines = contents.split("\n");
    var trie = makeTrie();
    for (var _i = 0, lines_1 = lines; _i < lines_1.length; _i++) {
        var l = lines_1[_i];
        for (var i = depth; i <= l.length; i++) {
            increment(trie, l.substr(i - depth, depth), 0);
        }
    }
    var mc = { trie: trie, depth: depth };
    fs.writeFileSync("/tmp/markov.json", JSON.stringify(runLengthEncodeMarkovChain(mc)));
    return mc;
}
exports.train = train;
function evaluate(mc, word) {
    var trie = mc.trie, depth = mc.depth;
    if (word.length < depth) {
        return 1;
    }
    var p = 1;
    for (var i = depth; i <= word.length; i++) {
        var cp = lookup(trie, word.substr(i - depth, depth), 0);
        if (cp === undefined) {
            cp = 0.0001;
        }
        p = p * cp;
    }
    return Math.pow(p, 1 / (word.length - depth + 1));
}
exports.evaluate = evaluate;
function testWord(mc, word) {
    console.log("\"" + word + "\": " + evaluate(mc, word));
}
function test(mc) {
    testWord(mc, "url");
    testWord(mc, "json");
    testWord(mc, "my_property");
    testWord(mc, "ordinary");
    testWord(mc, "different");
    testWord(mc, "189512");
    testWord(mc, "2BTZIqw0ntH9MvilQ3ewNY");
    testWord(mc, "0uBTNdNGb2OY5lou41iYL52LcDq2");
    testWord(mc, "-KpqHmWuDOUnr1hmAhxp");
    testWord(mc, "\ud83d\udebe \ud83c\udd92 \ud83c\udd93 \ud83c\udd95 \ud83c\udd96 \ud83c\udd97 \ud83c\udd99 \ud83c\udfe7");
}
exports.test = test;
function runLengthEncodeArray(arr) {
    var result = [];
    if (arr.length === 0)
        return result;
    var runItem = arr[0];
    var runStart = 0;
    var i = 1;
    while (i < arr.length) {
        var item = arr[i];
        if (item !== runItem) {
            result.push([i - runStart, runItem]);
            runItem = item;
            runStart = i;
        }
        i++;
    }
    result.push([i - runStart, runItem]);
    return result;
}
function runLengthEncodeTrie(t) {
    return {
        count: t.count,
        arr: runLengthEncodeArray(t.arr.map(function (x) {
            if (typeof x === "object") {
                return runLengthEncodeTrie(x);
            }
            return x;
        }))
    };
}
function runLengthEncodeMarkovChain(mc) {
    return { depth: mc.depth, trie: runLengthEncodeTrie(mc.trie) };
}
